/*
 * Copyright 2009 Prime Technology.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.primefaces.model;

import javax.faces.model.DataModel;

public class TreeModel extends DataModel {

	private Object wrappedData = null;	
	private int rowIndex = -1;
	private TreeNode root;
	
	public TreeModel() {}
	
	public TreeModel(TreeNode node) {
		this.wrappedData = node;
		this.root = node;
	}
	
	@Override
	public int getRowCount() {
		return ((TreeNode) wrappedData).getChildCount();
	}

	@Override
	public Object getRowData() {
		return ((TreeNode) wrappedData).getData();
	}

	@Override
	public int getRowIndex() {
		return rowIndex;
	}

	@Override
	public Object getWrappedData() {
		return wrappedData;
	}

	@Override
	public boolean isRowAvailable() {
		return false;
	}

	@Override
	public void setRowIndex(int rowIndex) {
		this.rowIndex = rowIndex;
		if(rowIndex >= 0) {
			TreeNode childNode = ((TreeNode) wrappedData).getChildren().get(rowIndex);
			setWrappedData(childNode);
		} else {
			setWrappedData(root);
		}
	}

	@Override
	public void setWrappedData(Object wrappedData) {
		this.wrappedData = wrappedData;
	}
}